﻿using App.VolgaIt.Dto;
using Domain.VolgaIt.AppSettings;
using Domain.VolgaIt.Entities;
using Infrastructure.VolgaIt;
using Infrastructure.VolgaIt.Context;
using Microsoft.Extensions.Options;
using Microsoft.IdentityModel.Tokens;
using System;
using System.Collections.Generic;
using System.IdentityModel.Tokens.Jwt;
using System.Linq;
using System.Security.Claims;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;

namespace App.VolgaIt.Services
{
    public class AccountService
    {
        private UnitOfWork _uow;
        private JwtOpt _jwtOpt;
        private PasswordHashOpt _passwordHashOpt;

        public AccountService(UnitOfWork uow, IOptions<JwtOpt> opt, IOptions<PasswordHashOpt> passwordHashOpt)
        {
            _uow = uow;
            _jwtOpt = opt.Value;
            _passwordHashOpt = passwordHashOpt.Value;
        }

        public async Task RegisterAdminAsync(RegisterDto dto)
        {
            var role = await _uow.UserRoleRepo.FindEntityByAsync(x => x.Name == "admin");
            await RegisterAsync(dto, role);
        }

        public async Task RegisterModeratorAsync(RegisterDto dto)
        {
            var role = await _uow.UserRoleRepo.FindEntityByAsync(x => x.Name == "moderator");
            await RegisterAsync(dto, role);
        }

        public async Task RegisterUserAsync(RegisterDto dto)
        {
            var role = await _uow.UserRoleRepo.FindEntityByAsync(x => x.Name == "user");
            await RegisterAsync(dto, role);
        }

        private async Task RegisterAsync(RegisterDto dto, UserRole role)
        {
            var user = await _uow.UserRepo.FindEntityByAsync(x => x.Login == dto.Login);
            if (user is not null)
                throw new Exception("такое имя уже занято");

            if (dto.Password == dto.Login)
                throw new Exception("пароль должен отличаться от имени");

            user = new User
            {
                Id = Guid.NewGuid(),
                Login = dto.Login,
                Password = GetProtectedPassword(dto.Password),
                Role = role,
            };

            _uow.UserRepo.AddEntity(user);
            await _uow.SaveChangesAsync();
        }

        public async Task<string> SignInAsync(SignInDto dto)
        {
            var user = await _uow.UserRepo.FindEntityByAsync(x => x.Login == dto.Login);
            if (user is null)
                throw new Exception("неверные данные для входа");

            if(!ChekPasswords(user.Password, dto.Password))
                throw new Exception("неверные данные для входа");

            return GetJwtToken(user);
        }

        private string GetJwtToken(User user)
        {
            var protectedSercret = new SymmetricSecurityKey(Encoding.ASCII.GetBytes(_jwtOpt.Secret));
;            var claims = new List<Claim>
            {
                new Claim("login", user.Login),
                new Claim(ClaimTypes.Role, user.Role.Name)
            };
            var claimsIdentities = new ClaimsIdentity(claims);

            var descriptor = new SecurityTokenDescriptor
            {
                Issuer = _jwtOpt.Issuer,
                Subject = claimsIdentities,
                Expires = DateTime.UtcNow.AddHours(_jwtOpt.ExpiresHours),
                SigningCredentials = new SigningCredentials(protectedSercret, SecurityAlgorithms.HmacSha256Signature)
            };
            var tokenHandler = new JwtSecurityTokenHandler();
            var token = tokenHandler.CreateToken(descriptor);

            return tokenHandler.WriteToken(token);
        }
        private string GetProtectedPassword(string password)
        {
            using var encrypt = new Rfc2898DeriveBytes(
                password: password,
                saltSize: _passwordHashOpt.SaltSize,
                iterations: _passwordHashOpt.Iterations,
                hashAlgorithm: HashAlgorithmName.SHA256);
            var salt = Convert.ToBase64String(encrypt.Salt);
            var key = Convert.ToBase64String(encrypt.GetBytes(_passwordHashOpt.KeySize));
            return $"{_passwordHashOpt.Iterations}.{salt}.{key}";
        }

        private bool ChekPasswords(string userPasssword, string password)
        {
            var passwordParts = userPasssword.Split('.');
            var userIteration = int.Parse(passwordParts[0]);
            var userSsalt = Convert.FromBase64String(passwordParts[1]);
            var userKey = passwordParts[2];

            using var encrypt = new Rfc2898DeriveBytes(
                password: password,
                salt: userSsalt,
                iterations: _passwordHashOpt.Iterations,
                hashAlgorithm: HashAlgorithmName.SHA256);

            var key = Convert.ToBase64String(encrypt.GetBytes(_passwordHashOpt.KeySize));
            return key == userKey;
        }
    }
}
